from typing import Generic, TypeVar

from rainbowadn.data.collection.trees.binary.binarytreecase import BinaryTreeCase
from rainbowadn.data.collection.trees.binary.binarytreecreationprotocol import BinaryTreeCreationProtocol

__all__ = ('BinaryTreeAction',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')
ActionType = TypeVar('ActionType')


class BinaryTreeAction(Generic[ActiveKeyType, MetaDataType, TreeType, ActionType]):
    async def on(
            self,
            protocol: BinaryTreeCreationProtocol[ActiveKeyType, MetaDataType, TreeType],
            tree: TreeType
    ) -> ActionType:
        if (split := await protocol.split(tree)) is None:
            return await self.on_null(protocol, tree)
        else:
            return await self.on_split(BinaryTreeCase(protocol, split, tree))

    async def on_null(
            self,
            protocol: BinaryTreeCreationProtocol[ActiveKeyType, MetaDataType, TreeType],
            tree: TreeType
    ) -> ActionType:
        raise NotImplementedError

    async def on_split(
            self,
            case: BinaryTreeCase[ActiveKeyType, MetaDataType, TreeType]
    ) -> ActionType:
        raise NotImplementedError
