from typing import Generic, TypeVar

from rainbowadn.core.hashpoint import HashPoint
from rainbowadn.data.collection.trees.binary.binarytreecase import BinaryTreeCase
from rainbowadn.data.collection.trees.binary.binarytreesplit import BinaryTreeSplit

__all__ = ('Symmetric', 'InnerOuter', 'OuterInner',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')


class Symmetric(
    Generic[ActiveKeyType, MetaDataType, TreeType]
):
    def __init__(
            self,
            case: BinaryTreeCase[
                ActiveKeyType, MetaDataType, TreeType
            ]
    ):
        self.case = case

    def inner(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        raise NotImplementedError

    def outer(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        raise NotImplementedError

    async def tree(
            self,
            inner: TreeType,
            outer: TreeType,
            key: HashPoint[ActiveKeyType]
    ) -> TreeType:
        raise NotImplementedError


class InnerOuter(Symmetric):
    def inner(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        return split.treel

    def outer(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        return split.treer

    async def tree(
            self,
            inner: TreeType,
            outer: TreeType,
            key: HashPoint[ActiveKeyType]
    ) -> TreeType:
        return await self.case.protocol.tree(inner, outer, key)


class OuterInner(Symmetric):
    def inner(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        return split.treer

    def outer(
            self,
            split: BinaryTreeSplit[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        return split.treel

    async def tree(
            self,
            inner: TreeType,
            outer: TreeType,
            key: HashPoint[ActiveKeyType]
    ) -> TreeType:
        return await self.case.protocol.tree(outer, inner, key)
