import abc
from typing import Generic, TypeVar

from rainbowadn.core.hashpoint import HashPoint
from rainbowadn.data.collection.trees.binary.binarytreebalancingprotocol import BinaryTreeBalancingProtocol
from rainbowadn.data.collection.trees.binary.binarytreecreationprotocol import BinaryTreeCreationProtocol

__all__ = ('BalancedTreeCreationProtocol',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')


class BalancedTreeCreationProtocol(
    BinaryTreeCreationProtocol[ActiveKeyType, MetaDataType, TreeType],
    Generic[ActiveKeyType, MetaDataType, TreeType],
    abc.ABC
):
    def __init__(
            self,
            protocol: BinaryTreeBalancingProtocol[ActiveKeyType, MetaDataType, TreeType]
    ):
        assert isinstance(protocol, BinaryTreeBalancingProtocol)
        self.protocol = protocol
        super().__init__(protocol.comparator)

    async def _tree(self, treel: TreeType, treer: TreeType, key: HashPoint[ActiveKeyType]) -> TreeType:
        raise NotImplementedError

    async def tree(self, treel: TreeType, treer: TreeType, key: HashPoint[ActiveKeyType]) -> TreeType:
        assert isinstance(key, HashPoint)
        return await self.protocol.balance(
            await self._tree(
                await self.protocol.balance(treel, self),
                await self.protocol.balance(treer, self),
                key
            ),
            self
        )
