from typing import Generic, TypeVar

from rainbowadn.core.hashpoint import HashPoint
from rainbowadn.data.collection.trees.binary.binarytreecreationprotocol import BinaryTreeCreationProtocol
from rainbowadn.data.collection.trees.comparison.comparator import Comparator

__all__ = ('BinaryTreeBalancingProtocol',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')


class BinaryTreeBalancingProtocol(Generic[ActiveKeyType, MetaDataType, TreeType]):
    def __init__(
            self,
            comparator: Comparator[ActiveKeyType]
    ):
        assert isinstance(comparator, Comparator)
        self.comparator = comparator

    def empty_metadata(self) -> HashPoint[MetaDataType]:
        raise NotImplementedError

    async def metadata(
            self,
            treel: TreeType,
            treer: TreeType,
            key: HashPoint[ActiveKeyType],
            protocol: BinaryTreeCreationProtocol[ActiveKeyType, MetaDataType, TreeType]
    ) -> HashPoint[MetaDataType]:
        raise NotImplementedError

    async def balance(
            self,
            tree: TreeType,
            protocol: BinaryTreeCreationProtocol[ActiveKeyType, MetaDataType, TreeType]
    ) -> TreeType:
        raise NotImplementedError
